#' Plot timeseries of attributed cases
#' 
#' Produce pdf document containing a plot showing the posterior expected number of cases attributed to each source in each year. For use with the temporal attribution model. The vertical bads indicate 95\% credible intervals.
#' @param o The output list from the function \link{sourceAttribution}.  
#' @param file Filename of the output. Put file=NULL to plot to the current graphics device.
#' @param burnin Number of samples to discard as burnin (plus 1).
#' @param col Vector of colours for the different sources.
#' @details Source labels are taken from the attribute \code{rownames(X)} and year labels are taken from \code{rownames(Y)}. 
#' @export
casesPlot<-function(o,file="cases.pdf",burnin=501,col=NULL) {
  srcs<-dim(o$X)[1]
  typs<-dim(o$X)[2]
  yrs<-dim(o$Y)[1]
  if (!is.null(file)) {pdf(file,width=8,height=6)}
  if (is.null(col)) {col<-c("darkorange","blue","black","red","darkgreen","purple","grey")}
  if (length(col)<srcs) {col<-c(col,srcs-rainbow(length(col)))}
  pts<-burnin:length(o$theta.stored)
  rgbs<-col2rgb(col)
  qcases<-array(NA,c(yrs,3,srcs))
  for (k in 1:yrs) {
    qcases[k,,]<-apply(o$cases[k,,pts],1,quantile,probs=c(0.025,0.5,0.975))
  }
  plot(1:yrs,t="n",xaxt="n",main="Number of human cases by source",ylim=c(0,max(qcases)),ylab="cases",xlab="",xlim=c(1,yrs))
  axis(1,at=1:yrs,labels=rownames(o$Y))
  for (i in 1:srcs) {
    polygon(c(1:yrs,rev(1:yrs)),c(qcases[,1,i],rev(qcases[,3,i])),col=rgb(rgbs[1,i],rgbs[2,i],rgbs[3,i],80,maxColorValue=255),border=NA)
    lines(1:yrs,qcases[,2,i],col=col[i],lwd=2)
  }
  legend("topleft",rownames(o$X),col=col[1:srcs],lwd=2,bg="white")
  if (!is.null(file)) {dev.off()}
}