#' Plot scatterplot giving an indication of model fit.
#' 
#' Produce pdf document containing a scatterplot showing the observed and expected number of cases. For use with the single attribution model. The shaded band indicates 95\% variation expected under the Poisson distribution.
#' @param o The output list from the function \link{sourceAttribution}.  
#' @param file Filename of the output.
#' @param burnin Number of samples to discard as burnin (plus 1).
#' @details This plot is often used in the source attribution literature to justify that the model is working, which it doesn't really do. It can be used to highlight issues (such as extra-Poisson variation), but the absence of issues here does not guarantee that there are no problems elsewhere. For example I would recommend checking that the relative prevalence posteriors look like their priors. 
#' @export
plotFitsFile<-function(o,file="fits.pdf",burnin=501) {
  srcs<-dim(o$X)[1]
  typs<-dim(o$X)[2]
  yrs<-dim(o$Y)[1]
  pts<-burnin:length(o$theta.stored)
  fits<-array(NA,c(yrs,typs,length(o$theta.stored)))
  for (k in pts) {
    if (length(dim(o$a.stored))==2) { # single attribution
      fits[,,k]<-rep(o$q.stored[,k]*(o$a.stored[,k]%*%o$r.stored[,,k]),each=yrs)
    } else { # temporal attribution
      fits[,,k]<-rep(o$q.stored[,k],each=yrs)*(o$a.stored[,,k]%*%o$r.stored[,,k])
    }
  }
  if (yrs>1) {
    qfits<-apply(fits[,,pts],c(1,2),quantile,probs=c(0.025,0.5,0.975))
  } else {
    qfits<-array(apply(fits[1,,pts],1,quantile,probs=c(0.025,0.5,0.975)),c(3,1,typs))
  }
  cat(dim(qfits),"\n")
  m<-max(o$Y)
  x<-0:100*m/100
  cols<-topo.colors(yrs)
  pdf(file,width=8,height=8)
  par(mfrow=c(1,1),mar=c(4, 4, 1, 1) + 0.1)
  plot(c(0,m),t="n",col="grey",xlab="Posterior median cases",ylab="Observed cases",main="",xlim=c(0,m),ylim=c(0,m))
  polygon(c(x,rev(x)),c(qpois(0.025,x),rev(qpois(0.975,x))),col="lightgrey",border=NA)
  lines(c(0,m),c(0,m),col="darkgrey")
  if (length(dim(o$a.stored))==2) {
    text(qfits[2,1,],m,colnames(o$X),cex=0.5,adj=c(1,0.25),srt=90)
  }
  segments(qfits[1,,],o$Y,qfits[3,,],o$Y,col=cols)
  points(qfits[2,,],o$Y,pch=16,col=cols)
  legend("bottomright",rownames(o$Y),fill=cols)
  dev.off()
}