% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sourceAttribution.r
\name{sourceAttribution}
\alias{sourceAttribution}
\title{Bayesian source attribution modelling}
\usage{
sourceAttribution(X, Y, temporal = TRUE, unobserved = FALSE, iters = 2500,
  thinning = 20 + (1 - temporal) * 80, r.prior = 1, theta.prior = c(1, 1),
  alpha = NULL, aSigma = diag(rep(0.005, srcs)), thetaSigma = 1,
  reps = NULL, adapt.reps = 0.05)
}
\arguments{
\item{X}{a sources x types matrix of source data (counts). The attributes \code{rownames(X)} and \code{colnames(X)} provide the labels to the sources and types respectively.}

\item{Y}{a years x types matrix of human data (counts). The attribute \code{rownames(Y)} provides the labels to the years.}

\item{temporal}{Run temporal attribution model (ie different attribution for each year)?}

\item{unobserved}{Include unobserved source?}

\item{iters}{Number of samples to draw from MCMC.}

\item{thinning}{Number of MCMC updates between samples. Default is 20 for temporal model and 100 for single attribution model.}

\item{r.prior}{Prior parameter for relative prevalence (\eqn{\gamma}).}

\item{theta.prior}{Vector of two parameters for precision of strain specific random effects (\eqn{\alpha^{(\theta)},\beta^{(\theta)}}).}

\item{alpha}{Prior parameters for \eqn{a}.}

\item{aSigma}{Initial covariance matrix for the joint proposals for \eqn{a} (on the log scale). This should adapt.}

\item{thetaSigma}{Initial proposal variance for \eqn{\theta} (adapts).}

\item{reps}{Initial number of repeats of the pairwise updates for \eqn{r} (adapts).}

\item{adapt.reps}{Minimum acceptance probability for \eqn{r} - if lower than this value \code{reps} is increased.}
}
\value{
list giving full details of MCMC input and output, including
\tabular{ll}{
 \code{X} \tab Source input matrix X\cr
 \code{Y} \tab Human case input matrix Y\cr
 \code{a.stored} \tab Array of MCMC samples for the source-specific factors \eqn{a}\cr
 \code{r.stored} \tab Array of MCMC samples for the relative prevalences \eqn{r}\cr
 \code{q.stored} \tab Array of MCMC samples for the strain-specific factors \eqn{q}\cr
 \code{theta.stored} \tab Matrix of MCMC samples for precision of the strain-specific random effects, \eqn{\theta}\cr
 \code{priors} \tab list of priors\cr
 \code{acceptance} \tab list of acceptance probabilities\cr
}
}
\description{
Attribute sources from relative prevalence data. This function fits a Poisson regression model using Markov chain Monte Carlo (MCMC) methods.
}
\examples{
data(campyNZ)
# thinning=100 will give better convergence, but makes the package check take a long time.
output <- sourceAttribution(campyNZ$X, campyNZ$Y, temporal=FALSE, thinning=1) 

# Examine output (in order of importance)

# casesPlot(output) # for temporal attribution
casesBarplot(output)  # for single attribution
plotPrevalenceFile(output) # histograms that look different from their prior (in red) indicate lack-of-fit
plotParameterFile(output) # check convergence
top(output)           # interesting which types appear in unobserved source
casesTable(output)    # useful summary for paper
plotFitsFile(output)  # points appearing outside grey band (Poisson variation) indicate lack-of-fit. NB converse does not hold!
}
\references{
Ahlstrom, C., Muellner, P., Spencer, S.E.F., \emph{et al.} 2017 Inferring source attribution from a multi-year multi-source
dataset of Salmonella in Minnesota. \emph{Zoonoses and public health}
}
